/***********************************************************
 * $Id: I18nHelper.java 42 2007-08-24 15:05:52Z wglas $
 * 
 * Tapestry Tools
 * 
 * Copyleft (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Created: 24.08.2007
 *
 * Author: wglas
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools;

import org.apache.tapestry.IComponent;
import org.apache.tapestry.IPage;
import org.xnap.commons.i18n.I18n;
import org.xnap.commons.i18n.I18nFactory;

/**
 * Static helper methods for gettext-commons handling inside
 * tapestry apps.
 * 
 * @author wglas
 */
public abstract class I18nHelper
{
    /**
     * Get the i18n object for the given page and the user's current locale.
     * 
     * @param page The page to get the translator for.
     * @return The i18n Object for the package of the implementing class of the page
     *         and the user's locale.
     */
    @SuppressWarnings("unchecked")
    public static I18n getI18n(IPage page) {
        
        Class clazz = page.getClass().getSuperclass();
        
        return I18nFactory.getI18n(clazz.getName(),
                                   "i18n.Messages", 
                                   clazz.getClassLoader(),
                                   page.getEngine().getLocale(),
                                   I18nFactory.DEFAULT);
    }
    
    /**
     * Get the i18n object for the given component and the user's current locale.
     * 
     * @param component The component to get the translator for.
     * @return The i18n Object for the package of the implementing class of the
     *         component and the user's locale.
     */
    @SuppressWarnings("unchecked")
    public static I18n getI18n(IComponent component) {
        
        Class clazz = component.getClass().getSuperclass();

        return I18nFactory.getI18n(clazz.getName(),
                                   "i18n.Messages", 
                                   I18n.class.getClassLoader(),
                                   component.getPage().getEngine().getLocale(),
                                   I18nFactory.DEFAULT);
    }
    
}
