/***********************************************************
 * $Id: I18n.java 50 2007-08-26 13:58:24Z lech $
 * 
 * Copyright (C) 2002-2006 ev-i Informationstechnologie GmbH
 *
 * Created: May 14, 2006
 *
 * Author: Wolfgang Glas/ev-i
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools.components;

import java.util.Collection;
import java.util.Locale;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.AbstractComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.annotations.ComponentClass;
import org.apache.tapestry.annotations.Parameter;
import org.clazzes.tapestry.tools.I18nHelper;

/**
 * @author wglas
 * 
 * This component inserts localized text into the html code.
 */
@ComponentClass(allowBody = false, allowInformalParameters = true)
public abstract class I18n extends AbstractComponent {
    /**
     * Contructs a new I8n instance.
     */
    public I18n() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.AbstractComponent#renderComponent(org.apache.tapestry.IMarkupWriter,
     *      org.apache.tapestry.IRequestCycle)
     */
    @SuppressWarnings("unchecked")
    @Override
    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle) {
        if (cycle.isRewinding())
            return;

        // collect data for i18n.
        String msg = getMsg();
        String nmsg = getNmsg();

        Object args = getArgs();
        Integer n = getN();

        Locale locale = cycle.getEngine().getLocale();

        org.xnap.commons.i18n.I18n i18n = I18nHelper.getI18n(cycle.getPage());

        // possibly open span for class attribute.
        boolean render = this.isParameterBound("class") || this.getRenderTag();

        if (render) {
            writer.begin(this.getTemplateTagName());
            renderInformalParameters(writer, cycle);
        }

        if (nmsg == null) {
            // singular only.
            if (args == null) {
                writer.print(i18n.tr(msg));
            } else if (args instanceof Object[]) {
                Object args_a[] = (Object[]) args;
                writer.print(String.format(locale, i18n.tr(msg), args_a));
            } else if (args instanceof Collection) {
                Collection<Object> c = (Collection<Object>) args;
                Object args_a[] = c.toArray();
                writer.print(String.format(locale, i18n.tr(msg), args_a));
            } else {
                writer.print(String.format(locale, i18n.tr(msg), args));
            }
        } else {
            // plural / singular forms.
            if (n == null)
                throw new ApplicationRuntimeException(Tapestry.format(
                        "I18n.argument-missing", "n"), this, getLocation(),
                        null);

            if (args == null) {
                writer.print(i18n.trn(msg, nmsg, n));
            } else if (args instanceof Object[]) {
                Object args_a[] = (Object[]) args;
                writer.print(String.format(locale, i18n.trn(msg, nmsg, n),
                        args_a));
            } else if (args instanceof Collection) {
                Collection<Object> c = (Collection<Object>) args;
                Object args_a[] = c.toArray();
                writer.print(String.format(locale, i18n.trn(msg, nmsg, n),
                        args_a));
            } else {
                writer.print(String
                        .format(locale, i18n.trn(msg, nmsg, n), args));
            }
        }

        if (render)
            writer.end();
    }

    /**
     * @return The message to be displayed. If this message contains JAVA format
     *         specifiers, the args property must be set.
     */
    @Parameter(required = false)
    public abstract String getMsg();

    /**
     * @return The plural message. This message is displayed, if the property n
     *         is set and the value of n is greater than 1.
     */
    @Parameter(required = false)
    public abstract String getNmsg();

    /**
     * @return The discriminator for plural messages. This value decides for the
     *         plural form to be used or not.
     */
    @Parameter(required = false)
    public abstract Integer getN();

    /**
     * @return The arguments to be used in order to fill the JAVA format
     *         specifiers in the message.
     */
    @Parameter(required = false)
    public abstract Object getArgs();

    /**
     * @return Whether to render the specified tag. The tag is rendered anyways, if
     *         a CSS class attribute is specified.
     */
    @Parameter(required = false, defaultValue = "true")
    public abstract boolean getRenderTag();
}
