/***********************************************************
 * $Id: LightDatePicker.java 29 2007-06-07 21:02:22Z lech $
 * 
 * Tapestry Tools
 * 
 * Copyleft (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Created: 05.06.2007
 *
 * Author: lech
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools.components;

import java.text.DateFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

import org.apache.tapestry.IAsset;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScript;
import org.apache.tapestry.PageRenderSupport;
import org.apache.tapestry.TapestryUtils;
import org.apache.tapestry.annotations.Asset;
import org.apache.tapestry.annotations.ComponentClass;
import org.apache.tapestry.annotations.InjectScript;
import org.apache.tapestry.annotations.Parameter;
import org.apache.tapestry.form.AbstractFormComponent;

/**
 * @author andreas
 * 
 */
@ComponentClass(allowInformalParameters=true,allowBody=false)
public abstract class LightDatePicker extends AbstractFormComponent {
  
  @Parameter
  public abstract String getFormat();
  
  @Parameter(defaultValue="true")
  public abstract boolean isReferenceRight();
  
  //@Parameter(defaultValue="false")
  //public abstract boolean isShowInfo();
  
  //@Parameter(defaultValue="false")
  //public abstract boolean isShowReferenceIcon();
  
  @Parameter(required=true)
  public abstract Date getValue();
  public abstract void setValue(Date value);
  
  //@Parameter
  //public abstract boolean isDisabled();
  
  @Parameter(defaultValue="true")
  public abstract boolean isIncludeWeek();
  
  @SuppressWarnings("unchecked")
  @Parameter
  public abstract Collection getHolidays();
  
  @Asset("/org/apache/tapestry/form/DatePickerIcon.png")
  public abstract IAsset getPopIcon();
  
  // /org/apache/tapestry/form/DatePickerIcon.png
  //@Asset("/org/apache/tapestry/form/DatePickerIcon.png")
  //public abstract IAsset getReferenceIcon();
  
  //@InjectAsset("infIcon")
  //public abstract IAsset getInfoIcon();
  
  public abstract IAsset getIcon();
  
  @Parameter(defaultValue="0")
  public abstract int getReviewInterval();
  
  private static final String SYM_NAME               = "name";
  
  private static final String SYM_FORMNAME           = "formName";
  
  private static final String SYM_MONTHNAMES         = "monthNames";
  
  private static final String SYM_SHORT_MONTHNAMES   = "shortMonthNames";
  
  private static final String SYM_WEEKDAYNAMES       = "weekDayNames";
  
  private static final String SYM_SHORT_WEEKDAYNAMES = "shortWeekDayNames";
  
  private static final String SYM_FIRSTDAYINWEEK     = "firstDayInWeek";
  
  private static final String SYM_MINDAYSINFIRSTWEEK = "minimalDaysInFirstWeek";
  
  private static final String SYM_FORMAT             = "format";
  
  private static final String SYM_INCL_WEEK          = "includeWeek";
  
  private static final String SYM_VALUE              = "value";
  
  private static final String SYM_INTERVAL           = "interval";
  
  private static final String SYM_HOLIDAYS           = "holidays";
  
  // Output symbol
  
  /*
   * protected void finishLoad() { super.finishLoad(); }
   */

  // Injections
  
  @InjectScript("form/LightDatePicker.script")
  public abstract IScript getScript();

  @SuppressWarnings("unchecked")
  protected void renderFormComponent(IMarkupWriter writer, IRequestCycle cycle) {
    
    String name = getName();
    
    String format= (getFormat()==null || getFormat().length()==0 ? "yyyy-MM-dd" : getFormat());
    
    SimpleDateFormat formatter = new SimpleDateFormat(format, getPage().getLocale());
    
    boolean disabled = isDisabled();
    //boolean showReference = isShowReferenceIcon();
    //boolean referenceRight = isReferenceRight();
    
      Locale locale = getPage().getLocale();
      DateFormatSymbols dfs = new DateFormatSymbols(locale);
      Calendar cal = Calendar.getInstance(locale);
      
      Date value = getValue();
      
      Map symbols = new HashMap();
      
      symbols.put(SYM_NAME, name);
      symbols.put(SYM_FORMAT, format);
      symbols.put(SYM_INCL_WEEK, isIncludeWeek() ? Boolean.TRUE
          : Boolean.FALSE);
      
      symbols.put(SYM_MONTHNAMES, makeStringList(dfs.getMonths(), 0, 12));
      symbols.put(SYM_SHORT_MONTHNAMES, makeStringList(dfs.getShortMonths(), 0,
          12));
      symbols.put(SYM_WEEKDAYNAMES, makeStringList(dfs.getWeekdays(), 1, 8));
      symbols.put(SYM_SHORT_WEEKDAYNAMES, makeStringList(
          dfs.getShortWeekdays(), 1, 8));
      symbols.put(SYM_FIRSTDAYINWEEK, new Integer(cal.getFirstDayOfWeek() - 1));
      symbols.put(SYM_MINDAYSINFIRSTWEEK, new Integer(cal
          .getMinimalDaysInFirstWeek()));
      symbols.put(SYM_FORMNAME, getForm().getName());
      symbols.put(SYM_VALUE, value);
      symbols.put(SYM_INTERVAL, getReviewInterval());
      
      if (getHolidays() != null) {
        Iterator it = getHolidays().iterator();
        int i = 0;
        int size = getHolidays().size();
        String[] dates = new String[size];
        
        while (it.hasNext()) {
          Calendar calendar = (Calendar) it.next();
          SimpleDateFormat dateForm = (SimpleDateFormat) SimpleDateFormat
              .getDateTimeInstance();
          dateForm.applyPattern("yyyy-MM-dd");
          dates[i] = dateForm.format(new Date(calendar.getTimeInMillis()));
          i++;
        }
        
        symbols.put(SYM_HOLIDAYS, makeStringList(dates, 0, dates.length));
      }
      else {
        symbols.put(SYM_HOLIDAYS, makeStringList(null, 0, 0));
      }
      //_script.execute(cycle, body, symbols);
      PageRenderSupport pageRenderSupport = TapestryUtils.getPageRenderSupport(cycle, this);
      getScript().execute(this, cycle, pageRenderSupport, symbols);
      
      // write the referenceDate part
      
      /*
      if (!referenceRight) { // place icons at the left side of input field
        if (!disabled && showReference) {
          writeReferenceLink(symbols, cycle, writer, false, name, false);
          writer.printRaw("&nbsp;");
        }
        else if (isShowInfo()) {
          writeReferenceLink(symbols, cycle, writer, false, name, true);
          writer.printRaw("&nbsp;");
        }
      }
      */
      
      // the input
      writer.beginEmpty("input");
      writer.attribute("type", "text");
      if (!disabled)
        writer.attribute("name", name);
      else
        writer.attribute("name", name + "dis");
      writer.attribute("title", formatter.toLocalizedPattern());
      
      String formattedDate = null;
      if (value != null) formattedDate = formatter.format(value);
      
      if (formattedDate != null) writer.attribute("value", formattedDate);
      
      if (disabled) writer.attribute("disabled", "disabled");
      
      renderInformalParameters(writer, cycle);
      
      if (disabled) {
        getForm().addHiddenValue(name, formattedDate);
      }
      
      writer.printRaw("&nbsp;");
      
      // the link for the popup
      if (!disabled) {
        writer.begin("a");
        writer.attribute("href", "#");
        writer.attribute("onclick",
            "calendarToggle(document.getElementsByName('" + name + "')[0]);");
        //IAsset icon = getIcon(); // getPopIcon()
        IAsset icon = getPopIcon(); // getPopIcon()
        
        writer.beginEmpty("img");
        //writer.attribute("src", icon.buildURL(cycle));
        writer.attribute("src", icon.buildURL());
        writer.attribute("border", 0);
        writer.attribute("style", "vertical-align:bottom");
        writer.end(); // <a>
      }
      
      // write the referenceDate part
      
      /*
      if (referenceRight) // place icons at the right side of input field
      {
        if (!disabled && showReference) {
          writer.printRaw("&nbsp;");
          writeReferenceLink(symbols, cycle, writer, true, name, false);
        }
        else if (isShowInfo()) {
          writer.printRaw("&nbsp;");
          writeReferenceLink(symbols, cycle, writer, true, name, true);
        }
      }
      */
    
      //String textValue = cycle.getRequestContext().getParameter(name);
      String textValue = cycle.getParameter(name);
      
      if (textValue==null || textValue.length()==0) return;
      
      try {
        Date date = formatter.parse(textValue);
        
        setValue(date);
      }
      catch (ParseException ex) {
      }
    
  }

  
  /**
   * Create a list of quoted strings. The list is suitable for initializing a
   * JavaScript array.
   */
  private String makeStringList(String[] a, int offset, int length) {
    StringBuffer b = new StringBuffer();
    for (int i = offset; i < length; i++) {
      // JavaScript is sensitive to some UNICODE characters. So for
      // the sake of simplicity, we just escape everything
      b.append('"');
      char[] ch = a[i].toCharArray();
      for (int j = 0; j < ch.length; j++) {
        if (ch[j] < 128) {
          b.append(ch[j]);
        }
        else {
          b.append(escape(ch[j]));
        }
      }
      
      b.append('"');
      if (i < length - 1) {
        b.append(", ");
      }
    }
    return b.toString();
  }
  
  /*
  @SuppressWarnings("unchecked")
  private void writeReferenceLink(Map symbols, IRequestCycle cycle,
      IMarkupWriter writer, boolean referenceRight,
      String dateFieldName, boolean justInfo) {
    writer.begin("a");
    writer.attribute("href", "#");
    
    if (justInfo) {
      writer.attribute("onmouseover",
          "return getDatePreview(infoReferenceDate('" + dateFieldName + "',"
              + getReviewInterval() + "));");
    }
    else {
      writer.attribute("onclick", "referenceDate('" + dateFieldName + "',"
          + getReviewInterval() + ")");
      
      if (referenceRight)
        writer.attribute("title", "Von rechtem Datum berechnen");
      else
        writer.attribute("title", "Von linkem Datum berechnen");
    }
    
    IAsset referenceIcon = getReferenceIcon();
    
    writer.beginEmpty("img");
    //writer.attribute("src", referenceIcon.buildURL(cycle));
    writer.attribute("src", referenceIcon.buildURL());
    writer.attribute("border", 0);
    
    writer.end(); // <a>
  }
  */
  
  /**
   * Create an escaped Unicode character
   * 
   * @param c
   * @return The unicode character in escaped string form
   */
  private static String escape(char c) {
    StringBuffer b = new StringBuffer();
    for (int i = 0; i < 4; i++) {
      b.append(Integer.toHexString(c & 0x000F).toUpperCase());
      c >>>= 4;
    }
    b.append("u\\");
    return b.reverse().toString();
  }
  
  /**
   * @see org.apache.tapestry.form.AbstractFormComponent#rewindFormComponent(org.apache.tapestry.IMarkupWriter,
   *      org.apache.tapestry.IRequestCycle)
   */
  protected void rewindFormComponent(IMarkupWriter writer, IRequestCycle cycle)
  {
      String value = cycle.getParameter(getName());
      try
      {
          String format= (getFormat()==null || getFormat().length()==0 ? "yyyy-MM-dd" : getFormat());
          SimpleDateFormat formatter = new SimpleDateFormat(format, getPage().getLocale());
          Date date = formatter.parse(value);
          setValue(date);
      }
      catch (ParseException p)
      { }
  }
  




}
