/***********************************************************
 * $Id: FormatTools.java 147 2009-09-25 10:02:50Z tatools $
 * 
 * Tapestry Tools
 * 
 * Copyright (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Created: 05.06.2007
 *
 * Author: lech
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools.util;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import org.apache.commons.lang.time.FastDateFormat;
import org.clazzes.util.datetime.UtcTimestamp;

/**
 * @author lech
 *
 */
public class FormatTools {
    
    protected FastDateFormat dateFormatter;
    protected FastDateFormat timeFormatter;
    protected FastDateFormat dateTimeFormatter;
    protected FastDateFormat localDateFormatter;
    protected FastDateFormat localTimeFormatter;
    protected FastDateFormat localDateTimeFormatter;

    
    protected void checkFormatters() {
        if (this.dateFormatter==null) {
            this.dateFormatter = FastDateFormat.getDateInstance(FastDateFormat.MEDIUM);
        }
        if (this.timeFormatter==null) {
            this.timeFormatter = FastDateFormat.getTimeInstance(FastDateFormat.SHORT);
        }
        if (this.dateTimeFormatter==null) {
            this.dateTimeFormatter = FastDateFormat.getDateTimeInstance(FastDateFormat.MEDIUM, FastDateFormat.SHORT);
        }
        if (this.localDateFormatter==null) {
            this.localDateFormatter = FastDateFormat.getDateInstance(FastDateFormat.MEDIUM, TimeZone.getDefault());
        }
        if (this.localTimeFormatter==null) {
            this.localTimeFormatter = FastDateFormat.getTimeInstance(FastDateFormat.SHORT, TimeZone.getDefault());
        }
        if (this.localDateTimeFormatter==null) {
            this.localDateTimeFormatter = FastDateFormat.getDateTimeInstance(FastDateFormat.MEDIUM, FastDateFormat.SHORT, TimeZone.getDefault());
        }
    }
    
    /**
     * Returns a snippet of a text, i.e. the first length chars plus ..., or the whole string if it is no longer than length.
     * @param s Text to snip of.
     * @param length max length of string to return.
     * @return The (beginning of or complete) string s with max. length characters.
     */
    public synchronized final String snipText(String s, int length) {
        if (s==null)
            return "";
        if (s.length() <= length)
            return s;
        return s.substring(0,length-4)+" ...";
    }

    /**
     * Returns a snippet of a text, i.e. the first 128 chars plus ..., or the whole string if it is no longer than 128.
     * @param s Text to snip of.
     * @return The (beginning of or complete) string s with max. 128 characters.
     */
    public synchronized final String snipText(String s) {
        return snipText(s, 128);
    }
    
    /**
     * @param ut
     * @return
     */
    public synchronized final String calendarToDate(UtcTimestamp ut) { 
        if (ut==null) 
            return "";
        return calendarToDate(ut.toCalendar());
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDate(Calendar calendar) {
        if (calendar==null) 
            return "";
        checkFormatters();
        return this.dateFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalDate(Calendar calendar) {
        if (calendar==null) 
            return "";
        checkFormatters();
        return this.localDateFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalDate(UtcTimestamp ut) {
        return calendarToLocalDate(ut.toCalendar());
    }
    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDate(Long millis) {
        if (millis==null) 
            return "";
        Calendar calendar=Calendar.getInstance();
        calendar.setTimeInMillis(millis);
        checkFormatters();
        return this.dateFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalDate(Long millis) {
        if (millis==null) 
            return "";
        Calendar calendar=Calendar.getInstance();
        calendar.setTimeInMillis(millis);
        checkFormatters();
        return this.localDateFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDate(Date date) {
        if (date==null) 
            return "";
        checkFormatters();
        return this.dateFormatter.format(date);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalDate(Date date) {
        if (date==null) 
            return "";
        checkFormatters();
        return this.localDateFormatter.format(date);
    }

    /**
     * @param ut
     * @return
     */
    public synchronized final String calendarToTime(UtcTimestamp ut) { 
        if (ut==null) 
            return "";
        return calendarToTime(ut.toCalendar());
    }

    /**
     * @param ut
     * @return
     */
    public synchronized final String calendarToLocalTime(UtcTimestamp ut) { 
        if (ut==null) 
            return "";
        return calendarToLocalTime(ut.toCalendar());
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToTime(Calendar calendar) {
        if (calendar==null) 
            return "";
        checkFormatters();
        return this.timeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalTime(Calendar calendar) {
        if (calendar==null) 
            return "";
        checkFormatters();
        return this.localTimeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToTime(Long millis) {
        if (millis==null) 
            return "";
        Calendar calendar=Calendar.getInstance();
        calendar.setTimeInMillis(millis);
        checkFormatters();
        return this.timeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalTime(Long millis) {
        if (millis==null) 
            return "";
        Calendar calendar=Calendar.getInstance();
        calendar.setTimeInMillis(millis);
        checkFormatters();
        return this.localTimeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToTime(Date date) {
        if (date==null) 
            return "";
        checkFormatters();
        return this.timeFormatter.format(date);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalTime(Date date) {
        if (date==null) 
            return "";
        checkFormatters();
        return this.localTimeFormatter.format(date);
    }

    /**
     * @param ut
     * @return
     */
    public synchronized final String calendarToDateTime(UtcTimestamp ut) { 
        if (ut==null) 
            return "";
        return calendarToDateTime(ut.toCalendar());
    }

    /**
     * @param ut
     * @return
     */
    public synchronized final String calendarToLocalDateTime(UtcTimestamp ut) { 
        if (ut==null) 
            return "";
        return calendarToLocalDateTime(ut.toCalendar());
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDateTime(Calendar calendar) {
        if (calendar==null) 
            return "";
        checkFormatters();
        return this.dateTimeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalDateTime(Calendar calendar) {
        if (calendar==null) 
            return "";
        checkFormatters();
        return this.localDateTimeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDateTime(Long millis) {
        if (millis==null) 
            return "";
        Calendar calendar=Calendar.getInstance();
        calendar.setTimeInMillis(millis);
        checkFormatters();
        return this.dateTimeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalDateTime(Long millis) {
        if (millis==null) 
            return "";
        Calendar calendar=Calendar.getInstance();
        calendar.setTimeInMillis(millis);
        checkFormatters();
        return this.localDateTimeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDateTime(Date date) {
        if (date==null) 
            return "";
        checkFormatters();
        return this.dateTimeFormatter.format(date);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToLocalDateTime(Date date) {
        if (date==null) 
            return "";
        checkFormatters();
        return this.localDateTimeFormatter.format(date);
    }
    
    public final String formatEuroAmount(Integer cents) {
        if (cents==null) 
            return "";
        return formatEuroAmount(cents, false);
    }
    
    public final String formatEuroAmount(Integer cents, boolean withEuro) {
        if (cents==null) 
            return "";
        StringBuffer sb=new StringBuffer();
        int euros=cents/100;
        sb.append(euros);
        sb.append(',');
        int euroCents=cents%100;
        sb.append((char)(euroCents / 10 + '0'));
        sb.append((char)(euroCents % 10 + '0'));
        if (withEuro) { 
            sb.append(" €");
        }
        return sb.toString();
    }

    public final String formatEuroAmount(Long cents) {
        if (cents==null) 
            return "";
        return formatEuroAmount(cents, false);
    }
    
    public final String formatEuroAmount(Long cents, boolean withEuro) {
        if (cents==null) 
            return "";
        StringBuffer sb=new StringBuffer();
        long euros=cents/100;
        sb.append(euros);
        sb.append(',');
        long euroCents=cents%100;
        sb.append((char)(euroCents / 10 + '0'));
        sb.append((char)(euroCents % 10 + '0'));
        if (withEuro) { 
            sb.append(" €");
        }
        return sb.toString();
    }

}
