/***********************************************************
 * $Id: BaseItemSelectionModel.java 109 2008-08-03 17:40:06Z lech $
 * 
 * Tapestry Tools
 * 
 * Copyright (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Created: 05.06.2007
 *
 * Author: lech
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools.util.form;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import org.apache.tapestry.form.IPropertySelectionModel;

/**
 * @author lech
 * 
 */
public class BaseItemSelectionModel<T extends SelectableItem> implements
        Serializable, IPropertySelectionModel {
    private static final long serialVersionUID = -7482675415024591607L;

    protected List<T> items;

    protected String nullValueText = "-";

    protected boolean allowNullValue;

    public BaseItemSelectionModel(List<T> itemList, boolean nullAllowed) {
        this.items = itemList;
        if (this.items==null)
            this.items=new Vector<T>();
        this.allowNullValue = nullAllowed;
    }

    public BaseItemSelectionModel(Collection<T> itemList, boolean nullAllowed) {
        this.items=new Vector<T>();
        if (itemList!=null)
            this.items.addAll(itemList);
        this.allowNullValue = nullAllowed;
    }

    public BaseItemSelectionModel(List<T> itemList) {
        this(itemList, false);
    }

    public BaseItemSelectionModel(Collection<T> itemList) {
        this(itemList, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getLabel(int)
     */
    public String getLabel(int index) {
        if (this.allowNullValue) {
            if (index == 0)
                return this.nullValueText;
            else
                return this.items.get(index - 1).getPrettyName();
        }
        return this.items.get(index).getPrettyName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getOption(int)
     */
    public Object getOption(int index) {
        if (this.allowNullValue) {
            if (index == 0)
                return null;
            else
                return this.items.get(index - 1);
        }
        return this.items.get(index);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getOptionCount()
     */
    public int getOptionCount() {
        if (this.allowNullValue)
            return this.items.size() + 1;
        else
            return this.items.size();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getValue(int)
     */
    public String getValue(int index) {
        return Integer.toString(index);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#translateValue(java.lang.String)
     */
    public Object translateValue(String value) {
        return getOption(Integer.parseInt(value));
    }

    public boolean isDisabled(int index) {
        return false;
    }

    /**
     * @return the nullValueText
     */
    public String getNullValueText() {
        return this.nullValueText;
    }

    /**
     * @param nullValueText the nullValueText to set
     */
    public void setNullValueText(String nullValueText) {
        this.nullValueText = nullValueText;
    }

}
