/***********************************************************
 * $Id: StringItemSelectionModel.java 106 2008-03-18 11:18:58Z lech $
 * 
 * Tapestry Tools
 * 
 * Copyright (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Created: 05.06.2007
 *
 * Author: lech
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools.util.form;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import org.apache.tapestry.form.IPropertySelectionModel;

/**
 * @author lech
 * 
 */
public class StringItemSelectionModel implements
        Serializable, IPropertySelectionModel {
    private static final long serialVersionUID = -7482675415024591607L;

    protected Vector<String> strings;

    protected String nullValueText = "-";

    protected boolean allowNullValue;

    public StringItemSelectionModel(List<String> stringList, boolean nullAllowed) {
        this.strings = new Vector<String>();
        if (stringList!=null)
            this.strings.addAll(stringList);
        this.allowNullValue = nullAllowed;
    }

    public StringItemSelectionModel(Collection<String> stringList, boolean nullAllowed) {
        this.strings = new Vector<String>();
        if (stringList!=null)
            this.strings.addAll(stringList);
        this.allowNullValue = nullAllowed;
    }

    public StringItemSelectionModel(List<String> stringList) {
        this(stringList, false);
    }

    public StringItemSelectionModel(Collection<String> stringList) {
        this(stringList, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getLabel(int)
     */
    public String getLabel(int index) {
        if (this.allowNullValue) {
            if (index == 0)
                return this.nullValueText;
            else
                return this.strings.get(index - 1);
        }
        return this.strings.get(index);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getOption(int)
     */
    public Object getOption(int index) {
        if (this.allowNullValue) {
            if (index == 0)
                return null;
            else
                return this.strings.get(index - 1);
        }
        return this.strings.get(index);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getOptionCount()
     */
    public int getOptionCount() {
        if (this.allowNullValue)
            return this.strings.size() + 1;
        else
            return this.strings.size();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#getValue(int)
     */
    public String getValue(int index) {
        return getLabel(index);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.form.IPropertySelectionModel#translateValue(java.lang.String)
     */
    public Object translateValue(String value) {
        return value;
    }

    public boolean isDisabled(int index) {
        return false;
    }

}
